import os, time
from os.path import *
import gnomeapplet, gtk, gtk.gdk, gconf, gobject
from gettext import gettext as _

import cohoba
from cohoba.ui.AccountsWindow import AccountsWindow
from cohoba.Utils import load_icon
from cohoba.telepathy.MergedContactList import get_contact_list, MergedContactList
from cohoba.telepathy.Contact import Contact
from cohoba.telepathy.ContactGroup import ContactGroup, FavoriteContactGroup, get_group_filter
from cohoba.ui.ContactListWindowManager import get_contact_list_window_manager
from cohoba.telepathy.ChannelHandler import get_channel_handler

class CohobaAppletMainWindow:
	def __init__(self):
		glade = gtk.glade.XML(join(cohoba.SHARED_DATA_DIR, "main.glade"), "main")
		self.main = glade.get_widget("main")
		self.main.hide()
				
		from cohoba.telepathy.MergedContactList import get_contact_list
		from cohoba.ui.ContactListView import ContactListView
		self.contacts = ContactListView(FavoriteContactGroup())
		self.contacts.show()
		glade.get_widget("contacts_scroll").add(self.contacts)
		
		self.message = glade.get_widget("message")		
		
		from cohoba.ui.PresenceComboBox import PresenceComboBox
		self.presence_combobox = PresenceComboBox(self.message)
		self.presence_combobox.show()
		self.presence_combobox.set_active(0)
		glade.get_widget("presence_table").pack_start(self.presence_combobox, False)
		
class CohobaAppletWidget(gtk.EventBox):
	def __init__(self, applet):
		gtk.EventBox.__init__(self)
		self.applet = applet
				
		image = gtk.Image()
		image.set_from_pixbuf(load_icon("cohoba.png"))
		self.add(image)
		
		self.main_ui = CohobaAppletMainWindow()
		
		self.applet.connect("button-press-event", self.on_applet_button_press)
		self.applet.setup_menu_from_file (
			cohoba.SHARED_DATA_DIR, "Cohoba.xml",
			None, [
			("About", self.on_about),
			("Accounts", self.on_accounts),
			])

	def make_menu(self, event):
		menu = gtk.Menu()
		menu.attach_to_widget(self, None)
		
		def contact_clicked(item, contact):
			get_channel_handler().start_conversation(contact)
			menu.popdown()
		
		def group_clicked(item, contact):
			get_contact_list_window_manager().show(contact)
			menu.popdown()
		
		label = gtk.Label()
		label.set_markup("Open Main Window")
		label.set_alignment(0.0, 0.0)
				
		image = gtk.Image()
		image.set_from_icon_name("stock_people", gtk.ICON_SIZE_DIALOG)

		box = gtk.HBox(False, 6)
		box.pack_start(image, expand=False)
		box.pack_start(label)

		item = gtk.MenuItem()
		item.add(box)
		item.connect('activate', lambda item: self.main_ui.main.present_with_time(gtk.get_current_event_time()))
		menu.append(item)
		
		menu.append(gtk.SeparatorMenuItem())
		
		for callback, klass, test in [(contact_clicked, Contact, lambda contact: True), (group_clicked, ContactGroup, lambda group: True)]:
			for contact, contact_description, pixbuf in [(row[MergedContactList.COL_CONTACT], row[MergedContactList.COL_STRING], row[MergedContactList.COL_PIXBUF]) for row in get_group_filter(FavoriteContactGroup(), get_contact_list()) if row[0].__class__ == klass and test(row[0])]:
			
				label = gtk.Label()
				label.set_markup(contact_description)
				label.set_alignment(0.0, 0.0)
						
				image = gtk.Image()
				image.set_from_pixbuf(pixbuf)

				box = gtk.HBox(False, 6)
				box.pack_start(image, expand=False)
				box.pack_start(label)

				item = gtk.MenuItem()
				item.add(box)		
				item.connect('activate', callback, contact)
				menu.append(item)
			
			if klass != ContactGroup:
				menu.append(gtk.SeparatorMenuItem())
		
		
		menu.show_all()
		menu.popup(None, None, None, event.button, event.time)

	def on_about (self, component, verb):
		print 'FIXME: to be implemnted', component, verb
	
	def on_accounts (self, component, verb):
		AccountsWindow().show()
								
	def on_applet_button_press(self, widget, event):
		if event.button == 1:
			self.make_menu(event)
#			self.main_ui.main.present_with_time(event.time)
			return True
			
		return False
