import gobject, gtk

import cohoba
from cohoba.telepathy.Constants import *
from os.path import join, splitext, exists

ICON_THEME = gtk.icon_theme_get_default()

def load_icon(icon, width=-1, height=-1):
	pixbuf = None
	if icon != None and icon != "":
		try:
			our_icon = join(cohoba.ART_DATA_DIR, icon)
			print our_icon
			if exists(our_icon):
				pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(our_icon, width, height)
			elif icon.startswith("/"):
				pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(icon, width, height)
			else:
				pixbuf = ICON_THEME.load_icon(splitext(icon)[0], width, gtk.ICON_LOOKUP_USE_BUILTIN)
		except Exception, msg1:
			try:
				pixbuf = ICON_THEME.load_icon(icon, width, gtk.ICON_LOOKUP_USE_BUILTIN)
			except Exception, msg2:
				print 'Error:load_icon:Icon Load Error:%s (or %s)' % (msg1, msg2)
				
	return pixbuf
	
def global_error_handler(e, msg=None):
	print 'Error callback:', msg, e

def presence_status_to_string(name, parameters={}):
	freetext = None
	if name == 'available':
		msg = 'Available'
	elif name == 'away':
		msg = 'Away'
	elif name == 'brb':
		msg = 'Be Right Back'
	elif name == 'busy':
		msg = 'Busy'
	elif name == 'dnd':
		msg = 'Do Not Disturb'
	elif name == 'xa':
		msg = 'Extended Away'
	elif name == 'hidden':
		msg = 'Hidden'
	elif name == 'offline':
		msg = 'Offline'
	else:
		msg = 'Unknown'
		name = 'offline'

	if "message" in parameters and parameters["message"] and parameters["message"] != "":
		freetext = parameters['message']

	return name, msg, freetext
		
def conn_status_to_string(status):
	if status == CONNECTION_STATUS_CONNECTED:
		return "Connected"
	elif status == CONNECTION_STATUS_CONNECTING:
		return "Connecting"
	elif status == CONNECTION_STATUS_DISCONNECTED:
		return "Disconnected"
	else:
		return "Unknown"
		
def conn_reason_to_string(reason):
	if reason == CONNECTION_STATUS_REASON_NONE_SPECIFIED:
		return "None"
	elif reason == CONNECTION_STATUS_REASON_REQUESTED:
		return "User Request"
	elif reason == CONNECTION_STATUS_REASON_NETWORK_ERROR:
		return "Network Error"
	elif reason == CONNECTION_STATUS_REASON_AUTHENTICATION_FAILED:
		return "Authentication Failed"
	elif reason == CONNECTION_STATUS_REASON_ENCRYPTION_ERROR:
		return "Encryption Error"
	else:
		return "Unknown"
           
import HTMLParser

class VCardParser(HTMLParser.HTMLParser):
	def __init__(self):
		HTMLParser.HTMLParser.__init__(self)
		self._reset()
		
	def _reset(self):
		self.keys = {}
		self.tag_stack = []
		self.chars = ""
		
	def parse(self, xvcard):
		self._reset()
		self.feed(xvcard)
		return self
	
	def handle_starttag(self, tag, attrs):
		self.tag_stack.append(tag.lower())
		self.chars = ""

	def handle_endtag(self, tag):
		tag = tag.lower()
		poptag = self.tag_stack.pop()
		if tag != poptag:
			print 'Error parsing the vcard, tag mismatch:', tag, poptag
			return
					
		d = self.keys
		for tag in self.tag_stack:
			d[tag] = d.get(tag, {})
			d = d[tag]
		self.chars = self.chars.strip()
		if self.chars != "":
			d[poptag] = self.chars
			
		self.chars = ""

	def handle_data(self, chars):
		self.chars = self.chars + chars
