import gobject, dbus

from cohoba.Utils import global_error_handler, presence_status_to_string, VCardParser
from cohoba.telepathy.TelepathyGroupMember import TelepathyGroupMember
from cohoba.telepathy.Constants import CONNECTION_HANDLE_TYPE_CONTACT, VCARD_KEY_NICKNAME, VCARD_KEY_FULLNAME
from cohoba.telepathy.DBusProxyProvider import get_proxy_provider

class Contact(TelepathyGroupMember):
	def __init__(self, conn_path, handle):
		TelepathyGroupMember.__init__(self, handle)
		self.vcard = None
		self.alias = None
		self.status = 'Offline'
		self.status_code = 'offline'
		self.status_message = None
		self.name = None
		self.avatar = None
		self.can_talk = True
		self.groups = []
		self.favorite = False
		self.conn_path = None
		self.handle = handle
		self.connect_to_connection(conn_path)
		
	def connect_to_connection(self, conn_path):
		self.conn_path = conn_path
		conn_iface = get_proxy_provider().get_iface(conn_path, 'org.freedesktop.Telepathy.Connection')
		conn_iface.InspectHandle(CONNECTION_HANDLE_TYPE_CONTACT, self.handle, reply_handler=self.on_handle_inspected, error_handler=global_error_handler)

		self.presence_iface = get_proxy_provider().get_iface(conn_path, 'org.freedesktop.Telepathy.Connection.Interface.Presence')
		self.presence_iface.connect_to_signal('PresenceUpdate', self.on_got_presence)
		self.presence_iface.RequestPresence([dbus.UInt32(self.handle)])
		
		self.alias_iface = get_proxy_provider().get_iface(conn_path, 'org.freedesktop.Telepathy.Connection.Interface.Aliasing')
		self.alias_iface.connect_to_signal('AliasesChanged', self.on_got_aliases)
		self.alias_iface.RequestAliases([dbus.UInt32(self.handle)], reply_handler=self.on_got_alias, error_handler=global_error_handler)	

		self.cinfo_iface = get_proxy_provider().get_iface(conn_path, 'org.freedesktop.Telepathy.Connection.Interface.ContactInfo')
		#self.cinfo_iface.connect_to_signal('GotContactInfo', self.on_got_contact_infos)
		#self.cinfo_iface.RequestContactInfo(dbus.UInt32(self.handle))
		
		self.avatar_iface = get_proxy_provider().get_iface(conn_path, 'org.freedesktop.Telepathy.Connection.Interface.Avatars')
		self.avatar_iface.connect_to_signal('AvatarUpdated', self.on_avatar_updated)
		# FIXME: Experimental support for avatars, uncomment the following line
		#self.on_got_avatar_token(["FIXME:token"])
		#self.avatar_iface.GetAvatarTokens([dbus.UInt32(self.handle)], reply_handler=self.on_got_avatar_token, error_handler=global_error_handler)
			
	def on_handle_inspected(self, name):
		self.name = name
		self.emit('updated')
		
	def on_got_presence(self, presences):
		if self.handle in presences:
			self.idle, statuses = presences[self.handle]

			for name, params in statuses.items():
				self.status_code, self.status, self.status_message = presence_status_to_string(name, params)
				break
			
			self.emit('updated')
				
	def on_got_contact_infos(self, handle, vcard):
		if handle != self.handle:
			return

		parser = VCardParser()
		parser.feed(vcard)
		self.vcard = parser.keys
		#print 'VCard parsed:', vcard, ':', self.vcard
		
		# Extract useful infos:
		self.alias = self.vcard.get(VCARD_KEY_NICKNAME, self.vcard.get(VCARD_KEY_FULLNAME, None))

		self.emit('updated')

	def on_got_alias(self, aliases):
		self.alias = aliases[0]
		self.emit('updated')

	def on_got_aliases(self, aliases):
		for handle, alias in aliases:
			if handle == self.handle:
				self.alias = alias
				self.emit('updated')
	
	def on_got_avatar_token(self, tokens):
		token = tokens[0]
		if token != "":
			self.avatar_iface.RequestAvatar(dbus.UInt32(self.handle), reply_handler=self.on_got_avatar, error_handler=global_error_handler)
			
	def on_got_avatar(self, data, mime):
		self.avatar = ''.join(chr(c) for c in data)
		self.emit('updated')
		
	def on_avatar_updated(self, handle, token):
		if handle == self.handle:
			self.on_got_avatar_token([token])			
		
	def __repr__(self):
		return 'Contact %d: %s/%s (%s[%s]-%s): %s' % (self.handle, self.name, self.alias, self.status, self.status_message, self.flag, self.vcard)

