import gobject, dbus

from cohoba.Utils import global_error_handler
from cohoba.telepathy.Constants import CONNECTION_HANDLE_TYPE_CONTACT,CHANNEL_TEXT_MESSAGE_TYPE_NORMAL
from cohoba.telepathy.DBusProxyProvider import get_proxy_provider

class TelepathyConversation(gobject.GObject):
	__gsignals__ = {
		"message-received" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [gobject.TYPE_STRING, gobject.TYPE_UINT]),
		"closed" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, []),
	}
	
	def __init__(self, conn_path, obj_path, contact):
		gobject.GObject.__init__(self)
		self.contact = contact
		
		self.conn_iface = get_proxy_provider().get_iface(conn_path, 'org.freedesktop.Telepathy.Connection')
		
		get_proxy_provider().create_proxy(get_proxy_provider().get_service(conn_path), obj_path)
		self.textchannel_iface = {
			"text": get_proxy_provider().get_iface(obj_path, "org.freedesktop.Telepathy.Channel.Type.Text"),
			"chan": get_proxy_provider().get_iface(obj_path, "org.freedesktop.Telepathy.Channel")}
		
		self.textchannel_iface["text"].connect_to_signal('Received', self.on_message_received)
		self.textchannel_iface["text"].connect_to_signal('Sent', self.on_message_sent)
		self.textchannel_iface["text"].ListPendingMessages(reply_handler=self.on_pending_messages_listed, error_handler=global_error_handler)
		
		self.textchannel_iface["chan"].connect_to_signal('Closed', self.on_channel_closed)
		
		self.media_channel = None
		
	def on_pending_messages_listed(self, pending_messages):
		for msg in pending_messages:
			self.on_message_received(*msg)
		
	def on_message_received(self, id, timestamp, sender, message_type, message):
		self.textchannel_iface["text"].AcknowledgePendingMessage(id, reply_handler=lambda: None, error_handler=global_error_handler)
		self.emit('message-received', message, message_type)
	
	def on_message_sent(self, timestamp, message_type, message):
		print 'Sent:', timestamp, message_type, message
	
	def on_channel_closed(self):
		self.emit('closed')
		
	#---------------------         
	def start_media(self, group):
		self.media_channel = group
		                       
	def send_message(self, message, message_type=CHANNEL_TEXT_MESSAGE_TYPE_NORMAL):
		self.textchannel_iface["text"].Send(message_type, message, reply_handler=lambda: None, error_handler=global_error_handler)
						
	def close(self):
		self.textchannel_iface["chan"].Close(reply_callback=lambda: None, error_callback=global_error_handler)
