import gtk, gtk.glade, gobject
from os.path import join
import cohoba

from cohoba.ui.AccountEditor import AccountEditor
from cohoba.ui.AccountsListView import AccountsListView
from cohoba.telepathy.AccountList import get_account_list
from cohoba.ui.ProtocolIcons import get_protocol_icons

class AccountsWindow(gtk.Window):
	COL_ICON = 0
	COL_AUTO = 1
	COL_ACCOUNT = 2
	
	def __init__(self):
		gtk.Window.__init__(self)		
		self.set_property("border_width", 12)
		self.set_property("window_position", gtk.WIN_POS_CENTER)
		
		glade = gtk.glade.XML(join(cohoba.SHARED_DATA_DIR, "accounts.glade"), "accounts_contents")#join(cohoba.SHARED_DATA_DIR, "conversation.glade")
		# Glade Widgets
		self.add(glade.get_widget("accounts_contents"))
		
		self.accounts = AccountsListView()
		self.accounts.show()
		glade.get_widget("accounts_scroll").add(self.accounts)
		
		self.new = glade.get_widget("new")
		self.remove = glade.get_widget("remove")
		self.close = glade.get_widget("close")
		self.properties = glade.get_widget("properties")
		self.connect_button = glade.get_widget("connect")
		self.disconnect_button = glade.get_widget("disconnect")
		
		self.close.connect("clicked", lambda w: self.destroy())
		self.new.connect("clicked", self.on_new)
		self.remove.connect("clicked", self.on_remove)
		self.properties.connect('clicked', self.on_properties)
		self.connect_button.connect('clicked', self.on_connect)
		self.disconnect_button.connect('clicked', self.on_disconnect)
		self.accounts.get_selection().connect('changed', self.on_selection_changed)
		self.accounts.get_model().connect('row-changed', self.on_row_changed)
			
	def on_new(self, widget):
		self.accounts.show_account_editor(get_account_list().add_account())
		
	def on_remove(self, widget):
		self.accounts.remove_selected()
	
	def on_properties(self, widget):
		self.accounts.show_selected_account_editor()
	
	def on_connect(self, widget):
		self.accounts.connect_selected(True)
		
	def on_disconnect(self, widget):
		self.accounts.connect_selected(False)
	
	def on_selection_changed(self, selection):
		model, iter = selection.get_selected()
		if iter != None:
			self.update_connect_buttons(iter)
		
	def on_row_changed(self, treemodel, path, iter):
		self.update_connect_buttons(iter)
		
	def update_connect_buttons(self, iter):
		if self.accounts.get_model() == None:
			# For some reasons when shutting down the program, the model is None..
			return
			
		account = self.accounts.get_model()[iter][0]
		self.connect_button.set_sensitive(account.is_connectable() and not account.is_connected)
		self.disconnect_button.set_sensitive(account.is_connected)
	
