import gtk
import gtk.glade
from os.path import join
import cohoba
from cohoba.ui.ConversationTextView import ConversationUpTextView
from cohoba.ui.PresenceIcons import get_presence_icons

class ConversationWindow(gtk.Window):
	NOTIFICATION_TYPING = 0
	
	def __init__(self, conversation):
		gtk.Window.__init__(self)		
		self.set_property("border_width", 12)
		self.set_property("window_position", gtk.WIN_POS_CENTER)
  
		glade = gtk.glade.XML(join(cohoba.SHARED_DATA_DIR, "conversation.glade"), "contents")
		# Glade Widgets
		self.add(glade.get_widget("contents"))
		self.view = ConversationUpTextView(conversation)
		self.view.show()
		glade.get_widget("conversationscroll").add(self.view)
		self.status = glade.get_widget("status")
		self.avatar = glade.get_widget("avatar")
		self.alias = glade.get_widget("alias")
		self.notificationicon = glade.get_widget("notificationicon") 
		self.notification = glade.get_widget("notification")
		self.talkbutton = glade.get_widget("talkbutton")
		
		self.hide_on_delete = True
		
		# Widgets signals
		self.connect('delete-event', self.on_window_delete)
		self.talkbutton.connect('toggled', self.on_talk_toggled)
		conversation.connect('closed', self.on_conversation_closed)
		
		# Default Values
		self.set_alias(None, "Unknown")
		self.set_status(None)
		self.set_avatar(None, None)
		self.set_notification(None)
		
		self.conversation = conversation
		self.conversation.connect('message-received', self.on_message_received)
		
		self.conversation.contact.connect('updated', self.on_contact_updated)
		self.on_contact_updated(conversation.contact)
		
	def on_contact_updated(self, contact):
		self.set_title(contact.name)
		self.set_alias(contact.alias, contact.name)
		self.set_avatar(contact.status_code, contact.avatar)
		self.set_status(contact.status, contact.status_message)
			
		if contact.can_talk:
			self.talkbutton.show()
	
	def on_message_received(self, conversation, message, message_type):
		self.present_with_time(gtk.get_current_event_time())
	
	def on_conversation_closed(self, conversation):
		self.hide_on_delete = False
		
	def on_window_delete(self, win, event):
		self.hide()
		return self.hide_on_delete
	
	def on_talk_toggled(self, button):
		from cohoba.telepathy.ChannelHandler import get_channel_handler
		get_channel_handler().start_media(self.conversation.contact)
	
	def start_media(self, group):
		print 'Starting media conversation from chan handler'
		self.conversation.start_media(group)
        
	def set_status(self, status, msg=None):
		if status == None and msg == None:
			self.status.hide()
			return

		template = "<span size='small'>%s</span>"			
		if status != None and msg != None:
			print 'Gaaah', status, msg
			template = template % ("%s\n%s" % (status, msg))
		elif status != None:
			template = template % status
		elif msg != None:
			template = template % msg
			
		self.status.set_markup(template)
		self.status.show()
	
	def set_notification(self, notification):
		if notification == self.NOTIFICATION_TYPING:
			self.notificationicon.set_from_stock(gtk.STOCK_EDIT, gtk.ICON_SIZE_MENU)
			self.notification.set_markup("<span size='small' style='italic'>%s</span>" % "Composing a Message")
		else:
			self.notificationicon.hide()
			self.notification.hide()
			return
			
		self.notificationicon.show()
		self.notification.show()
		
	def set_avatar(self, presence, avatar):
		if presence == None:
			self.avatar.set_property("icon-name", "stock_person")
			self.avatar.set_property("icon-size", gtk.ICON_SIZE_DIALOG)
		else:
			self.avatar.set_from_pixbuf(get_presence_icons(presence, avatar)[1])
		
	def set_alias(self, alias, name):
		if alias == None:
			alias = name
			name = ""
			
		self.alias.set_markup("<span size='large' weight='ultrabold'>%s</span><span weight='ultrabold'>\n%s</span>" % (alias, name))
